@@ -1,6 +1,6 @@
 # abstract_select.py
 #
-# Copyright 2018-2021 Romain F. T.
+# Copyright 2018-2022 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ import cairo
 from gi.repository import Gtk
 from .abstract_tool import AbstractAbstractTool
 from .optionsbar_selection import OptionsBarSelection
+from .utilities_colors import utilities_gdk_rgba_to_normalized_array
 from .utilities_overlay import utilities_show_overlay_on_context
 from .selection_manager import NoSelectionPixbufException
 
@@ -43,12 +44,15 @@ class AbstractSelectionTool(AbstractAbstractTool):
 	############################################################################
 	# UI implementations #######################################################
 
-	def get_edition_status(self):
+	def get_options_label(self):
+		return _("Selection")
+
+	def get_editing_tips(self):
 		if self.selection_is_active():
-			label = _("Drag the selection or right-click on the canvas")
+			label_tip = _("Drag the selection or right-click on the canvas")
 		else:
-			label = _("Select an area or right-click on the canvas")
-		return label
+			label_tip = _("Select an area or right-click on the canvas")
+		return [label_tip]
 
 	def get_options_model(self):
 		builder = Gtk.Builder.new_from_resource(self.UI_PATH + 'selection.ui')
@@ -98,7 +102,7 @@ class AbstractSelectionTool(AbstractAbstractTool):
 	def press_define(self, event_x, event_y):
 		pass # implemented by actual tools
 
-	def motion_define(self, event_x, event_y):
+	def motion_define(self, event_x, event_y, render):
 		pass # implemented by actual tools
 
 	def release_define(self, surface, event_x, event_y):
@@ -122,10 +126,10 @@ class AbstractSelectionTool(AbstractAbstractTool):
 			self.restore_pixbuf()
 			self.non_destructive_show_modif()
 
-	def on_motion_on_area(self, event, surface, event_x, event_y):
+	def on_motion_on_area(self, event, surface, event_x, event_y, render=True):
 		if self.behavior == 'define':
-			self.motion_define(event_x, event_y)
-		elif self.behavior == 'drag':
+			self.motion_define(event_x, event_y, render)
+		elif render and self.behavior == 'drag':
 			self._preview_drag_to(event_x, event_y)
 
 	def _preview_drag_to(self, event_x, event_y):
@@ -157,7 +161,7 @@ class AbstractSelectionTool(AbstractAbstractTool):
 		elif self.behavior == 'drag':
 			self._apply_drag_to(event_x, event_y)
 
-	def on_draw(self, area, ccontext):
+	def on_draw_above(self, area, ccontext):
 		if not self.selection_is_active():
 			return
 		ldx = self.local_dx
@@ -165,7 +169,8 @@ class AbstractSelectionTool(AbstractAbstractTool):
 		self.get_selection().show_selection_on_surface(ccontext, True, ldx, ldy)
 		dragged_path = self.get_selection().get_path_with_scroll(ldx, ldy)
 		# ^ Method not really use elsewhere, could it be private?
-		utilities_show_overlay_on_context(ccontext, dragged_path, True)
+		thickness = self.get_overlay_thickness()
+		utilities_show_overlay_on_context(ccontext, dragged_path, thickness)
 
 	############################################################################
 	# Pre-loading the selection manager with non-essential data ################
@@ -192,10 +197,10 @@ class AbstractSelectionTool(AbstractAbstractTool):
 		is used in `self.select_all` (abstract, here), and in the "rectangle
 		selection" implementation."""
 		# XXX could be in the selection manager?
-		x0 = int( min(press_x, release_x) )
-		y0 = int( min(press_y, release_y) )
-		x1 = int( max(press_x, release_x) )
-		y1 = int( max(press_y, release_y) )
+		x0 = min(press_x, release_x)
+		y0 = min(press_y, release_y)
+		x1 = max(press_x, release_x)
+		y1 = max(press_y, release_y)
 		w = x1 - x0
 		h = y1 - y0
 		if w <= 0 or h <= 0:
@@ -257,6 +262,17 @@ class AbstractSelectionTool(AbstractAbstractTool):
 		self.apply_operation(operation)
 		self.operation_type = 'op-define'
 
+	def replace_canvas(self):
+		self.operation_type = 'op-replace-canvas'
+		# XXX it's copied so many times?! ugly but seems necessary
+		self._future_pixbuf = self.get_selection_pixbuf().copy()
+		operation = self.build_operation()
+		self.apply_operation(operation)
+		self.operation_type = 'op-define'
+
+	def invert_selection(self):
+		pass # TODO
+
 	def unselect_and_apply(self):
 		# Pre-loading the coords is NEEDED because we may "unselect_and_apply" a
 		# selection which was defined by a selection manager in very different
@@ -288,10 +304,10 @@ class AbstractSelectionTool(AbstractAbstractTool):
 		# FIXME the result is questionable when there was alpha in the area...
 		if replacement_type == 'initial':
 			gdk_rgba = self.get_image().get_initial_rgba()
-			color = [gdk_rgba.red, gdk_rgba.green, gdk_rgba.blue, gdk_rgba.alpha]
+			color = utilities_gdk_rgba_to_normalized_array(gdk_rgba)
 		elif replacement_type == 'secondary':
 			gdk_rgba = self.window.options_manager.get_right_color()
-			color = [gdk_rgba.red, gdk_rgba.green, gdk_rgba.blue, gdk_rgba.alpha]
+			color = utilities_gdk_rgba_to_normalized_array(gdk_rgba)
 		else: # == 'alpha':
 			color = [1.0, 1.0, 1.0, 0.0]
 		# XXX the replacement is performed when the selection is defined, which
@@ -325,6 +341,14 @@ class AbstractSelectionTool(AbstractAbstractTool):
 		self.get_selection().set_coords(False, op['pixb_x'], op['pixb_y'])
 		self.get_selection().set_pixbuf(op['pixbuf'].copy())
 
+	def _op_replace_canvas(self, op):
+		if op['pixbuf'] is None:
+			raise NoSelectionPixbufException()
+		self.get_image().set_main_pixbuf(op['pixbuf'].copy())
+		self.get_image().use_stable_pixbuf()
+		self.get_selection().reset(True)
+		self.get_selection().reset_future_data()
+
 	def _op_clean(self, operation):
 		if operation['initial_path'] is None:
 			return # The user double-clicked: there is no path, and it's normal
@@ -374,6 +398,9 @@ class AbstractSelectionTool(AbstractAbstractTool):
 			# de type "clic-droit > importer" ou "clic-droit > coller".
 			# On charge un pixbuf dans le selection_manager.
 			self._op_import(operation)
+		elif operation['operation_type'] == 'op-replace-canvas':
+			# TODO commentaire
+			self._op_replace_canvas(operation)
 		elif operation['operation_type'] == 'op-define':
 			# Opération instantanée (sans preview), correspondant à une
 			# sélection (rectangulaire ou non) par définition d'un path.
