@@ -1,21 +1,6 @@
-# history_manager.py
-#
-# Copyright 2018-2021 Romain F. T.
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Licensed under GPL3 https://github.com/maoschanz/drawing/blob/master/LICENSE
 
-from gi.repository import Gdk, Gio, GdkPixbuf
+from gi.repository import Gdk, Gio, GdkPixbuf, GLib
 # from .abstract_tool import WrongToolIdException
 
 ################################################################################
@@ -29,6 +14,7 @@ class DrHistoryManager():
 		self._undo_history = []
 		self._redo_history = []
 		self._is_saved = True
+		self._waiting_for_rebuild = False
 
 	def get_saved(self):
 		# XXX undoing/redoing doesn't update the title so the "*" isn't visible
@@ -60,20 +46,20 @@ class DrHistoryManager():
 		if len(self._undo_history) > 0:
 			last_op = self._undo_history.pop()
 			self._redo_history.append(last_op)
-		self._rebuild_from_history()
+		self._rebuild_from_history_async()
+		self._image.update_history_sensitivity()
 
 	def try_redo(self, *args):
 		operation = self._redo_history.pop()
 		if operation['tool_id'] is None:
 			self._undo_history.append(operation)
-			self._image.restore_first_pixbuf()
+			self._image.restore_last_state()
 		else:
 			self._get_tool(operation['tool_id']).apply_operation(operation)
 
 	def can_undo(self):
 		# XXX incorrect si ya des states et qu'on redo
 		return (len(self._undo_history) > 0) or self._operation_is_ongoing()
-		# rarely called while an operation is ongoing so that ^ is stupid
 
 	def can_redo(self):
 		# XXX incorrect si ya des states ?
@@ -98,6 +84,13 @@ class DrHistoryManager():
 #			redo_label = redoable_action[0]['tool_id']
 #		self._image.window.update_history_actions_labels(undo_label, redo_label)
 
+	def rewind_history(self):
+		"""Put the entire 'undo' history into the 'redo' history, so the image
+		can be reset without losing any data."""
+		self._redo_history = self._undo_history[::-1] + self._redo_history
+		self._undo_history = []
+		self._image.update_history_sensitivity()
+
 	############################################################################
 	# Serialized operations ####################################################
 
@@ -122,7 +115,7 @@ class DrHistoryManager():
 		self.initial_operation = {
 			'tool_id': None,
 			'pixbuf': pixbuf,
-			'red': r, 'green': g, 'blue': b, 'alpha': a,
+			'rgba': Gdk.RGBA(red=r, green=g, blue=b, alpha=a),
 			'width': width, 'height': height
 		}
 
@@ -171,10 +164,26 @@ class DrHistoryManager():
 	############################################################################
 	# Other private methods ####################################################
 
-	def _rebuild_from_history(self):
-		"""Rebuild the image according to the content of the current history."""
+	def _rebuild_from_history_async(self):
+		if not self._waiting_for_rebuild:
+			# No need to duplicate calls, it will be rebuilt soon anyway
+			self._waiting_for_rebuild = True
+			GLib.timeout_add(500, self._rebuild_from_history, {})
+		# This introduces an artificial delay of half a second, BUT in the case
+		# where an user undoes several operations, i expect between 2 and 4
+		# presses on ctrl+z during these 500ms, so there will be between 2 and 4
+		# times less recomputation.
+
+	def _rebuild_from_history(self, async_cb_data={}):
+		"""Rebuild the image according to the content of the current history.
+		This is used as a GSourceFunc so it should return False."""
+		if not self._waiting_for_rebuild:
+			# It has already been rebuild by an other async call
+			return False
+		self._waiting_for_rebuild = False
+
 		last_save_index = self._get_last_state_index(True)
-		self._image.restore_first_pixbuf()
+		self._image.restore_last_state()
 		history = self._undo_history.copy()
 		self._undo_history = []
 		for op in history:
@@ -185,7 +194,7 @@ class DrHistoryManager():
 				# print("skip", op['tool_id'])
 				self._undo_history.append(op)
 		self._image.update()
-		self._image.update_history_sensitivity()
+		return False
 
 	def _operation_is_ongoing(self):
 		return self._image.active_tool().has_ongoing_operation()
@@ -195,7 +204,7 @@ class DrHistoryManager():
 		if tool_id in all_tools:
 			return all_tools[tool_id]
 		else:
-			self._image.window.prompt_message(True, _("Error: no tool '%s'" % tool_id))
+			self._image.window.reveal_action_report(_("Error: no tool '%s'" % tool_id))
 			# no raise: this may happen normally if last_save_index is incorrect
 
 	############################################################################
