@@ -1,6 +1,6 @@
 # image.py
 #
-# Copyright 2018-2021 Romain F. T.
+# Copyright 2018-2022 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,11 +17,10 @@
 
 import cairo, random, math
 from gi.repository import Gtk, Gdk, Gio, GdkPixbuf, Pango, GLib
-from .gi_composites import GtkTemplate
 from .history_manager import DrHistoryManager
 from .selection_manager import DrSelectionManager
 from .properties import DrPropertiesDialog
-from .utilities import InvalidFileFormatException
+from .utilities_files import InvalidFileFormatException
 from .utilities_overlay import utilities_generic_canvas_outline
 
 class DrMotionBehavior():
@@ -39,26 +38,51 @@ class NoPixbufNoChangeException(Exception):
 
 ################################################################################
 
-@GtkTemplate(ui='/com/github/maoschanz/drawing/ui/image.ui')
+@Gtk.Template(resource_path='/com/github/maoschanz/drawing/ui/image.ui')
 class DrImage(Gtk.Box):
 	__gtype_name__ = 'DrImage'
 
-	_drawing_area = GtkTemplate.Child()
-	_h_scrollbar = GtkTemplate.Child()
-	_v_scrollbar = GtkTemplate.Child()
+	_drawing_area = Gtk.Template.Child()
+	_h_scrollbar = Gtk.Template.Child()
+	_v_scrollbar = Gtk.Template.Child()
+	reload_info_bar = Gtk.Template.Child()
+	reload_label = Gtk.Template.Child()
 
+	# HiDPI scale factor
 	SCALE_FACTOR = 1.0 # XXX doesn't work well enough to be anything else
 
+	# Threshold between normal rendering and crisp (costly) rendering
+	ZOOM_THRESHOLD = 4.0
+
+	# Maximal level of zoom (crisp rendering only)
+	ZOOM_MAX = 2000
+
 	def __init__(self, window, **kwargs):
 		super().__init__(**kwargs)
-		self.init_template()
 		self.window = window
 
 		self.gfile = None
 		self.filename = None
-		self._fps_counter = 0
-		self.reset_fps_counter()
-		self._framerate_hint = 1.0
+		self._waiting_for_monitor = False
+		self._gfile_monitor = None
+		self._can_reload()
+
+		# Closing the info bar
+		self.reload_info_bar.connect('close', self.hide_reload_message)
+		self.reload_info_bar.connect('response', self.hide_reload_message)
+
+		# Framerate limit
+		self._rendering_is_locked = False
+		self._framerate_hint = 0
+
+		self._ctrl_pressed = False
+
+		if self.window.devel_mode:
+			# Framerate tracking (debug only)
+			self._skipped_frames = 0
+			self._fps_counter = 0
+			if self.window.should_track_framerate:
+				self.reset_fps_counter()
 
 		self._init_drawing_area()
 
@@ -138,35 +162,42 @@ class DrImage(Gtk.Box):
 	def init_background(self, width, height, background_rgba):
 		self.init_image_common()
 		self._history.set_initial_operation(background_rgba, None, width, height)
-		self.restore_first_pixbuf()
+		self.restore_last_state()
 
 	def try_load_pixbuf(self, pixbuf):
 		self.init_image_common()
 		self._load_pixbuf_common(pixbuf)
-		self.restore_first_pixbuf()
+		self.restore_last_state()
 		self.update_title()
 
 	def _new_blank_pixbuf(self, w, h):
 		return GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, w, h)
 
-	def restore_first_pixbuf(self):
+	def restore_last_state(self):
 		"""Set the last saved pixbuf from the history as the main_pixbuf. This
 		is used to rebuild the picture from its history."""
 		last_saved_pixbuf_op = self._history.get_last_saved_state()
+		self._apply_state(last_saved_pixbuf_op)
 
+	def reset_to_initial_pixbuf(self):
+		self._apply_state(self._history.initial_operation)
+		self._history.rewind_history()
+
+	def _apply_state(self, state_op):
 		# restore the state found in the history
-		pixbuf = last_saved_pixbuf_op['pixbuf']
-		width = last_saved_pixbuf_op['width']
-		height = last_saved_pixbuf_op['height']
+		pixbuf = state_op['pixbuf']
+		width = state_op['width']
+		height = state_op['height']
 		self.set_temp_pixbuf(self._new_blank_pixbuf(1, 1))
 		self.selection.init_pixbuf()
 		self.surface = cairo.ImageSurface(cairo.Format.ARGB32, width, height)
 		if pixbuf is None:
 			# no pixbuf in the operation: the restored state is a blank one
-			r = last_saved_pixbuf_op['red']
-			g = last_saved_pixbuf_op['green']
-			b = last_saved_pixbuf_op['blue']
-			a = last_saved_pixbuf_op['alpha']
+			rgba = state_op['rgba']
+			r = rgba.red
+			g = rgba.green
+			b = rgba.blue
+			a = rgba.alpha
 			self.set_main_pixbuf(self._new_blank_pixbuf(width, height))
 			cairo_context = cairo.Context(self.surface)
 			cairo_context.set_source_rgba(r, g, b, a)
@@ -174,10 +205,11 @@ class DrImage(Gtk.Box):
 			self.update()
 			self.set_surface_as_stable_pixbuf()
 		else:
-			self.set_main_pixbuf(last_saved_pixbuf_op['pixbuf'].copy())
+			self.set_main_pixbuf(state_op['pixbuf'].copy())
 			self.use_stable_pixbuf()
 
 	############################################################################
+	# (re)loading the pixbuf of a given file ###################################
 
 	def _load_pixbuf_common(self, pixbuf):
 		if not pixbuf.get_has_alpha():
@@ -190,15 +222,11 @@ class DrImage(Gtk.Box):
 	def reload_from_disk(self):
 		"""Safely reloads the image from the disk."""
 		if self.gfile is None:
-			# XXX no, the action shouldn't be active in the first place
-			isnt_saved = not self.window.saving_manager.confirm_save_modifs()
-			if isnt_saved or self.get_file_path() is None:
-				self.window.prompt_message(True, \
-				            _("Can't reload a never-saved file from the disk."))
-				return
+			# the action shouldn't be active in the first place
+			return
 		disk_pixbuf = GdkPixbuf.Pixbuf.new_from_file(self.get_file_path())
 		self._load_pixbuf_common(disk_pixbuf)
-		self.window.set_picture_title(self.update_title())
+		self.window.update_picture_title()
 		self.use_stable_pixbuf()
 		self.update()
 		self.remember_current_state()
@@ -207,17 +235,44 @@ class DrImage(Gtk.Box):
 		try:
 			self.gfile = gfile
 			pixbuf = GdkPixbuf.Pixbuf.new_from_file(self.get_file_path())
+			self.connect_gfile_monitoring()
 		except Exception as ex:
 			if not ex.message:
 				ex.message = "[exception without a valid message]"
 			ex = InvalidFileFormatException(ex.message, gfile.get_path())
-			self.window.prompt_message(True, ex.message)
+			self.window.reveal_action_report(ex.message)
 			self.gfile = None
 			pixbuf = self._new_blank_pixbuf(100, 100)
 			# XXX dans l'idéal on devrait ne rien ouvrir non ? ou si besoin (si
 			# ya pas de fenêtre) ouvrir un truc respectant les settings, plutôt
 			# qu'un petit pixbuf corrompu
 		self.try_load_pixbuf(pixbuf)
+		self._can_reload()
+
+	def connect_gfile_monitoring(self):
+		flags = Gio.FileMonitorFlags.WATCH_MOUNTS
+		self._gfile_monitor = self.gfile.monitor(flags)
+		self._gfile_monitor.connect('changed', self.reveal_reload_message)
+
+	def lock_monitoring(self):
+		self._waiting_for_monitor = True
+
+	def reveal_reload_message(self, *args):
+		if self._waiting_for_monitor:
+			# I'm not sure this lock is 100% correct because i'm monitoring the
+			# portal proxy file when testing with flatpak. Better than nothing.
+			if args[3] != Gio.FileMonitorEvent.CHANGED:
+				self._waiting_for_monitor = False
+			return
+		self._can_reload()
+		self.reload_label.set_visible(self.window.get_allocated_width() > 500)
+		self.reload_info_bar.set_visible(True)
+
+	def hide_reload_message(self, *args):
+		self.reload_info_bar.set_visible(False)
+
+	def _can_reload(self):
+		self.set_action_sensitivity('reload_file', self.gfile is not None)
 
 	############################################################################
 	# Image title and tab management ###########################################
@@ -291,6 +346,10 @@ class DrImage(Gtk.Box):
 	def show_properties(self):
 		DrPropertiesDialog(self.window, self)
 
+	def update_image_wide_actions(self):
+		self.update_history_sensitivity()
+		self._can_reload()
+
 	############################################################################
 	# History management #######################################################
 
@@ -320,12 +379,7 @@ class DrImage(Gtk.Box):
 		return not self._history.has_initial_pixbuf()
 
 	def get_initial_rgba(self):
-		operation = self._history.initial_operation
-		r = operation['red']
-		g = operation['green']
-		b = operation['blue']
-		a = operation['alpha']
-		return Gdk.RGBA(red=r, green=g, blue=b, alpha=a)
+		return self._history.initial_operation['rgba']
 
 	############################################################################
 	# Misc ? ###################################################################
@@ -350,24 +404,16 @@ class DrImage(Gtk.Box):
 	def active_tool(self):
 		return self.window.active_tool()
 
+	def get_mouse_is_pressed(self):
+		return self._is_pressed
+
 	############################################################################
 	# Drawing area, main pixbuf, and surface management ########################
 
-	def reset_fps_counter(self, async_cb_data={}):
-		"""Development only: live-display the evolution of the framerate of the
-		drawing area. The max should be around 60, but many tools don't require
-		so many redraws."""
-		if self.window.should_track_framerate:
-			# Context: this is a debug information that users will never see
-			self.window.prompt_message(True, _("%s frames per second") % self._fps_counter)
-			self._fps_counter = 0
-			GLib.timeout_add(1000, self.reset_fps_counter, {})
-		else:
-			self.window.prompt_message(False, "")
-
 	def on_draw(self, area, cairo_context):
 		"""Signal callback. Executed when self._drawing_area is redrawn."""
-		self._fps_counter += 1
+		if self.window.devel_mode:
+			self._fps_counter += 1
 
 		# Background color
 		cairo_context.set_source_rgba(*self._bg_rgba)
@@ -379,6 +425,8 @@ class DrImage(Gtk.Box):
 		# Image (with scroll position)
 		cairo_context.set_source_surface(self.get_surface(), \
 		                                 -1 * self.scroll_x, -1 * self.scroll_y)
+		if self.is_zoomed_surface_sharp():
+			cairo_context.get_source().set_filter(cairo.FILTER_NEAREST)
 		cairo_context.paint()
 
 		# What the tool shows on the canvas, upon what it paints, for example an
@@ -386,8 +434,9 @@ class DrImage(Gtk.Box):
 		self.active_tool().on_draw_above(area, cairo_context)
 
 		# Limit of the canvas (for readability)
-		utilities_generic_canvas_outline(cairo_context, self.get_pixbuf_width(), \
-		                              self.get_pixbuf_height(), self.zoom_level)
+		utilities_generic_canvas_outline(cairo_context, self.zoom_level, \
+		                              self.get_pixbuf_width() - self.scroll_x, \
+		                             self.get_pixbuf_height() - self.scroll_y)
 
 	def on_press_on_area(self, area, event):
 		"""Signal callback. Executed when a mouse button is pressed on
@@ -406,8 +455,11 @@ class DrImage(Gtk.Box):
 			self._slip_init_y = self.scroll_y
 			return
 		self.motion_behavior = DrMotionBehavior.DRAW
-		self.active_tool().on_press_on_area(event, self.surface, event_x, event_y)
 		self._is_pressed = True
+		self.window.set_window_subtitles()
+		# subtitles must be generated *before* calling the tool, otherwise any
+		# property changed by a modifier would be reset by `get_editing_tips`
+		self.active_tool().on_press_on_area(event, self.surface, event_x, event_y)
 
 	def on_motion_on_area(self, area, event):
 		"""Signal callback. Executed when the mouse pointer moves upon
@@ -415,6 +467,7 @@ class DrImage(Gtk.Box):
 		If a button (not the mouse wheel) is pressed, the tool's method should
 		have an effect on the image, otherwise it shouldn't change anything
 		except the mouse cursor icon for example."""
+		event_x, event_y = self.get_event_coords(event)
 
 		if self.motion_behavior == DrMotionBehavior.HOVER:
 			# Some tools need the coords in the image, others need the coords on
@@ -423,9 +476,15 @@ class DrImage(Gtk.Box):
 
 		elif self.motion_behavior == DrMotionBehavior.DRAW:
 			# implicitely impossible if not self._is_pressed
-			event_x, event_y = self.get_event_coords(event)
-			self.active_tool().on_motion_on_area(event, self.surface, event_x, event_y)
-			self.update(True)
+			self.active_tool().on_motion_on_area(event, self.surface, event_x, \
+			                                 event_y, self._rendering_is_locked)
+			if self._rendering_is_locked:
+				if self.window.devel_mode:
+					self._skipped_frames += 1
+				return
+			self._rendering_is_locked = True
+			self.update()
+			GLib.timeout_add(self._framerate_hint, self._async_unlock, {})
 
 		else: # self.motion_behavior == DrMotionBehavior.SLIP:
 			self.scroll_x = self._slip_init_x
@@ -434,10 +493,24 @@ class DrImage(Gtk.Box):
 			delta_y = self._slip_press_y - event.y
 			self.add_deltas(delta_x, delta_y, 1 / self.zoom_level)
 
+		# If <Ctrl> is pressed, a tooltip displaying contextual information is
+		# shown: by default, it contains at least the pointer coordinates.
+		if (event.state & Gdk.ModifierType.CONTROL_MASK) == Gdk.ModifierType.CONTROL_MASK:
+			self._ctrl_pressed = True
+		if self._ctrl_pressed:
+			full_tooltip_text = str(event_x) + ", " + str(event_y)
+			tool_tooltip = self._get_tool_tooltip(event_x, event_y)
+			if tool_tooltip is not None:
+				full_tooltip_text += "\n" + tool_tooltip
+			self._drawing_area.set_tooltip_text(full_tooltip_text)
+		else:
+			self._drawing_area.set_tooltip_text(None)
+
 	def on_release_on_area(self, area, event):
 		"""Signal callback. Executed when a mouse button is released on
 		self._drawing_area, if the button is not the signal is transmitted to
 		the selected tool."""
+		self._ctrl_pressed = False
 		if self.motion_behavior == DrMotionBehavior.SLIP:
 			if not self._is_slip_moving():
 				self.window.on_middle_click()
@@ -446,8 +519,9 @@ class DrImage(Gtk.Box):
 		self.motion_behavior = DrMotionBehavior.HOVER
 		event_x, event_y = self.get_event_coords(event)
 		self.active_tool().on_release_on_area(event, self.surface, event_x, event_y)
-		self.window.set_picture_title()
 		self._is_pressed = False
+		self.window.update_picture_title() # just to add the star
+		self.window.set_window_subtitles() # the tool's state changed
 
 	def _is_slip_moving(self):
 		"""Tells if the pointer moved while the middle button of the mouse is
@@ -456,12 +530,17 @@ class DrImage(Gtk.Box):
 		my = abs(self._slip_init_y - self.scroll_y) > DrMotionBehavior._LIMIT
 		return mx or my
 
-	def update(self, allow_imperfect=False):
-		if allow_imperfect and random.random() > self._framerate_hint:
-			pass
-		else:
-			# print('image.py: _drawing_area.queue_draw')
-			self._drawing_area.queue_draw()
+	def _get_tool_tooltip(self, ev_x, ev_y):
+		return self.active_tool().get_tooltip(ev_x, ev_y ,self.motion_behavior)
+
+	def update(self):
+		# print('image.py: _drawing_area.queue_draw')
+		self._drawing_area.queue_draw()
+
+	def _async_unlock(self, content_params={}):
+		"""This is used as a GSourceFunc so it should return False."""
+		self._rendering_is_locked = False
+		return False
 
 	def get_surface(self):
 		return self.surface
@@ -480,16 +559,18 @@ class DrImage(Gtk.Box):
 		w = self.surface.get_width()
 		h = self.surface.get_height()
 		self.main_pixbuf = Gdk.pixbuf_get_from_surface(self.surface, 0, 0, w, h)
-		# self._framerate_hint = math.sqrt(500000 / (w * h))
-		# print("hint :", self._framerate_hint)
+		self._framerate_hint = math.sqrt(w * h) - 1000
+		self._framerate_hint = int(self._framerate_hint * 0.2)
+		# between 500 and 33ms (= between 2 and 30 fps)
+		self._framerate_hint = max(33, min(500, self._framerate_hint))
+		# print("image.py: hint =", self._framerate_hint)
 
-	def use_stable_pixbuf(self, allow_imperfect=False):
-		if allow_imperfect and random.random() > self._framerate_hint:
-			pass
-		else:
-			# print('image.py: use_stable_pixbuf')
-			# maybe the "scale" parameter should be 1 instead of 0
-			self.surface = Gdk.cairo_surface_create_from_pixbuf(self.main_pixbuf, 0, None)
+	def use_stable_pixbuf(self):
+		"""This is called by tools' `restore_pixbuf`, so at the beginning of
+		each operation (even unapplied)."""
+		# maybe the "scale" parameter should be 1 instead of 0
+		self.surface = Gdk.cairo_surface_create_from_pixbuf(self.main_pixbuf, 0, None)
+		# print('image.py: use_stable_pixbuf')
 		self.surface.set_device_scale(self.SCALE_FACTOR, self.SCALE_FACTOR)
 
 	def get_pixbuf_width(self):
@@ -520,6 +601,26 @@ class DrImage(Gtk.Box):
 		self.use_stable_pixbuf()
 		self.update()
 
+	############################################################################
+	# Framerate tracking (debug only) ##########################################
+
+	def reset_fps_counter(self, async_cb_data={}):
+		"""Development only: live-display the evolution of the framerate of the
+		drawing area. The max should be around 60, but many tools don't require
+		so many redraws.
+		This is used as a GSourceFunc so it should return False."""
+		if self.window.should_track_framerate:
+			# Context: this is a debug information that users will never see
+			msg = _("%s frames per second") % self._fps_counter
+			msg += " (" + str(self._skipped_frames) + " motion inputs skipped)"
+			self.window.reveal_message(msg)
+			self._fps_counter = 0
+			self._skipped_frames = 0
+			GLib.timeout_add(1000, self.reset_fps_counter, {})
+		elif self.window.info_bar.get_visible():
+			self.window.reveal_message("Tracking stopped.", True)
+		return False
+
 	############################################################################
 	# Interaction with the minimap #############################################
 
@@ -529,7 +630,7 @@ class DrImage(Gtk.Box):
 	def get_widget_height(self):
 		return self._drawing_area.get_allocated_height()
 
-	def get_mini_pixbuf(self, preview_size):
+	def generate_mini_pixbuf(self, preview_size):
 		mpb_width = self.get_pixbuf_width()
 		mpb_height = self.get_pixbuf_height()
 		if mpb_height > mpb_width:
@@ -540,7 +641,7 @@ class DrImage(Gtk.Box):
 			mh = preview_size * (mpb_height/mpb_width)
 		return self.main_pixbuf.scale_simple(mw, mh, GdkPixbuf.InterpType.TILES)
 
-	def get_show_overlay(self):
+	def get_minimap_need_overlay(self):
 		mpb_width = self.get_pixbuf_width()
 		mpb_height = self.get_pixbuf_height()
 		show_x = self.get_widget_width() < mpb_width * self.zoom_level
@@ -559,13 +660,34 @@ class DrImage(Gtk.Box):
 	############################################################################
 	# Scroll and zoom levels ###################################################
 
+	def get_previewed_width(self):
+		# Indirect way to know if it's a transform tool
+		if self.active_tool().menu_id == 1:
+			if not self.active_tool().apply_to_selection:
+				return self.temp_pixbuf.get_width() + 12
+		return self.get_pixbuf_width()
+
+	def get_previewed_height(self):
+		# Indirect way to know if it's a transform tool
+		if self.active_tool().menu_id == 1:
+			if not self.active_tool().apply_to_selection:
+				return self.temp_pixbuf.get_height() + 12
+		return self.get_pixbuf_height()
+
 	def fake_scrollbar_update(self):
 		self.add_deltas(0, 0, 0)
 
-	def get_event_coords(self, event):
+	def get_event_coords(self, event, as_integers=True):
 		event_x = self.scroll_x + (event.x / self.zoom_level)
 		event_y = self.scroll_y + (event.y / self.zoom_level)
-		return event_x, event_y
+		if as_integers:
+			# `int()` will truncate to the lower integer so we need this to get
+			# an accurate behavior when doing pixel-art for example
+			event_x += 0.5
+			event_y += 0.5
+			return int(event_x), int(event_y)
+		else:
+			return event_x, event_y
 
 	def get_corrected_coords(self, x1, x2, y1, y2, with_selection, with_zoom):
 		"""Do whatever coordinates conversions are needed by tools like `crop`
@@ -596,16 +718,16 @@ class DrImage(Gtk.Box):
 			x1 = 0
 			y1 = 0
 		# width_left, width_right, height_top, height_bottom
-		wl, wr, ht, hb = self.get_corrected_coords(x1, width, y1, height, \
-		                                               apply_to_selection, True)
-		# FIXME using local deltas this way "works" but isn't mathematically
+		wl, wr, ht, hb = self.get_corrected_coords(int(x1), width, int(y1), \
+		                                       height, apply_to_selection, True)
+		# XXX using local deltas this way "works" but isn't mathematically
 		# correct: scaled selections have a "null" and excentred central nineth
-		# ^ c'est vrai ça ??
+		# ^ c'est toujours vrai ça ??
 		wl += 0.4 * width * self.zoom_level
 		wr -= 0.4 * width * self.zoom_level
 		ht += 0.4 * height * self.zoom_level
 		hb -= 0.4 * height * self.zoom_level
-		return wl, wr, ht, hb
+		return {'wl': wl, 'wr': wr, 'ht': ht, 'hb': hb}
 
 	def on_scroll_on_area(self, area, event):
 		# TODO https://lazka.github.io/pgi-docs/index.html#Gdk-3.0/classes/EventScroll.html#Gdk.EventScroll
@@ -613,17 +735,36 @@ class DrImage(Gtk.Box):
 		if ctrl_is_used == self._ctrl_to_zoom:
 			self._zoom_to_point(event)
 		else:
-			self.add_deltas(event.delta_x, event.delta_y, 10)
+			acceleration = 20 / self._zoom_profile()
+			self.add_deltas(event.delta_x, event.delta_y, acceleration)
 
 	def on_scrollbar_value_change(self, scrollbar):
 		self.correct_coords(self._h_scrollbar.get_value(), self._v_scrollbar.get_value())
-		self.update()
+		self.update() # allowing imperfect framerate would likely be useless
+
+	def reset_deltas(self, delta_x, delta_y):
+		if delta_x > 0:
+			wanted_x = self.get_previewed_width()
+		elif delta_x < 0:
+			wanted_x = 0
+		else:
+			wanted_x = self.scroll_x
+
+		if delta_y > 0:
+			wanted_y = self.get_previewed_height()
+		elif delta_y < 0:
+			wanted_y = 0
+		else:
+			wanted_y = self.scroll_y
+
+		self.correct_coords(wanted_x, wanted_y)
+		self.window.minimap.update_overlay()
 
 	def add_deltas(self, delta_x, delta_y, factor):
 		wanted_x = self.scroll_x + int(delta_x * factor)
 		wanted_y = self.scroll_y + int(delta_y * factor)
 		self.correct_coords(wanted_x, wanted_y)
-		self.window.minimap.update_minimap(False)
+		self.window.minimap.update_overlay()
 
 	def correct_coords(self, wanted_x, wanted_y):
 		available_w = self.get_widget_width()
@@ -632,13 +773,13 @@ class DrImage(Gtk.Box):
 			return # could be better handled
 
 		# Update the horizontal scrollbar
-		mpb_width = self.get_pixbuf_width()
+		mpb_width = self.get_previewed_width()
 		wanted_x = min(wanted_x, self.get_max_coord(mpb_width, available_w))
 		wanted_x = max(wanted_x, 0)
 		self.update_scrollbar(False, available_w, int(mpb_width), int(wanted_x))
 
 		# Update the vertical scrollbar
-		mpb_height = self.get_pixbuf_height()
+		mpb_height = self.get_previewed_height()
 		wanted_y = min(wanted_y, self.get_max_coord(mpb_height, available_h))
 		wanted_y = max(wanted_y, 0)
 		self.update_scrollbar(True, available_h, int(mpb_height), int(wanted_y))
@@ -666,55 +807,55 @@ class DrImage(Gtk.Box):
 		pointer stays as much as possible under the pointer."""
 		event_x, event_y = self.get_event_coords(event)
 
-		zoom_delta = (event.delta_x + event.delta_y) * -4 # Arbitrary
+		# Updating the zoom level
+		zoom_delta = (event.delta_x + event.delta_y) * -1 * self._zoom_profile()
 		self.inc_zoom_level(zoom_delta)
 
-		# Where the zoom event occurs, in terms of percentages, in the widget's
-		# reference frame. Like, "at 10% of the width, and 80% of the height".
-		proportion_w = event.x / self.get_widget_width()
-		proportion_h = event.y / self.get_widget_height()
-
-		# Size of the rectangle between the current scroll position (values of
-		# the self.scroll_* attributes) and the zoom event. Both are expressed
-		# in pixels in the reference frame of the pixbuf.
-		corner_w = event_x - self.scroll_x
-		corner_h = event_y - self.scroll_y
-
-		# Dark magic based on unsound experiments XXX
-		fake_delta_x = corner_w * (proportion_w - 0.5)
-		fake_delta_y = corner_h * (proportion_h - 0.5)
-		if zoom_delta > 0:
-			# Zooming in
-			fake_delta_x *= 0.5
-			fake_delta_y *= 0.5
-		else:
-			# Zooming out
-			fake_delta_x *= -0.1
-			fake_delta_y *= -0.1
+		new_event_x, new_event_y = self.get_event_coords(event)
+		delta_correction_x = event_x - new_event_x
+		delta_correction_y = event_y - new_event_y
 
 		# Updating the scroll position based on the values previously found
-		self.add_deltas(fake_delta_x, fake_delta_y, 1)
+		self.add_deltas(delta_correction_x, delta_correction_y, 1)
 
 	def inc_zoom_level(self, delta):
 		self.set_zoom_level((self.zoom_level * 100) + delta)
 
 	def set_zoom_level(self, level):
-		normalized_zoom_level = max(min(level, 400), 20)
+		normalized_zoom_level = max(min(level, self.ZOOM_MAX), 20)
 		self.zoom_level = (int(normalized_zoom_level)/100)
 		self.window.minimap.update_zoom_scale(self.zoom_level)
+		if self.is_zoomed_surface_sharp():
+			self.window.minimap.set_zoom_label(self.zoom_level * 100)
 		self.fake_scrollbar_update()
 		self.update()
 
 	def set_opti_zoom_level(self, *args):
 		allocated_width = self.get_widget_width()
 		allocated_height = self.get_widget_height()
-		h_ratio = allocated_width / self.get_pixbuf_width()
-		v_ratio = allocated_height / self.get_pixbuf_height()
+		h_ratio = allocated_width / self.get_previewed_width()
+		v_ratio = allocated_height / self.get_previewed_height()
 		opti = min(h_ratio, v_ratio) * 99 # Not 100 because some margin is cool
 		self.set_zoom_level(opti)
 		self.scroll_x = 0
 		self.scroll_y = 0
 
+	def is_zoomed_surface_sharp(self):
+		return self.zoom_level > self.ZOOM_THRESHOLD
+
+	def _zoom_profile(self):
+		"""This is the 'speed' of the zoom scrolling: when between 20% and 100%
+		it's quite precise, but between 1000% and 2000% we prefer being as fast
+		as possible."""
+		if self.zoom_level < 2.0:
+			return 3.0
+		elif self.zoom_level < 4.0:
+			return 6.0
+		elif self.zoom_level < 10.0:
+			return 10.0
+		else:
+			return 20.0
+
 	############################################################################
 ################################################################################
 
