@@ -1,6 +1,6 @@
 # options_manager.py
 #
-# Copyright 2018-2021 Romain F. T.
+# Copyright 2018-2022 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import GLib, Gio
+from gi.repository import GLib, Gio, Gdk
 
 class DrOptionsManager():
 	__gtype_name__ = 'DrOptionsManager'
@@ -29,6 +29,7 @@ class DrOptionsManager():
 		self._active_pane_id = None
 		self._boolean_actions_from_gsetting = {}
 		self._string_actions_from_gsetting = {}
+		self._mirror_lock = False
 
 	def _action_exists(self, name):
 		return self.window.lookup_action(name) is not None
@@ -73,7 +74,7 @@ class DrOptionsManager():
 		new_value = args[1].get_boolean()
 
 		args[0].set_state(GLib.Variant.new_boolean(new_value))
-		self.window.set_picture_title()
+		self.window.set_window_subtitles()
 		self.get_active_pane().hide_options_menu()
 
 	def _enum_callback(self, *args):
@@ -88,9 +89,46 @@ class DrOptionsManager():
 
 		# Actually change the state to the new value.
 		args[0].set_state(GLib.Variant.new_string(new_value))
-		self.window.set_picture_title()
+		self.window.set_window_subtitles()
 		self.get_active_pane().hide_options_menu()
 
+	############################################################################
+	# Special case: mirrorable actions are required when using radio-buttons ###
+
+	def mirrored_action_callback(self, mirror_action_name, *args):
+		"""Actions whose callbacks use this method can be used in menus. It sets
+		the lock required to avoid infinite recursion caused by the
+		synchronisation with a mirror action used on GtkRadioButtons."""
+		self._enum_callback(*args)
+		mirroring_action = self.window.lookup_action(mirror_action_name)
+		if args[1].get_string() == mirroring_action.get_state().get_string():
+			return False
+		self._mirror_lock = True
+		self._enum_callback(mirroring_action, args[1])
+		self._mirror_lock = False
+		return True
+
+	def mirroring_action_callback(self, mirrored_action_name, *args):
+		"""Actions whose callbacks use this method should NEVER be added to any
+		menu. Such an action should mirror the action provided as an argument
+		(which can be added to menus).
+		Those actions are intended to be used by GtkRadioButtons, whose weird
+		behaviors include sending a `change-state` signal when being unchecked,
+		thus triggering this callback twice. So a synchronisation mechanism is
+		needed, with a lock & an other "classic" action duplicating the data."""
+		if self._mirror_lock:
+			return False
+		if args[1].get_string() == args[0].get_state().get_string():
+			return False
+		mirrored_action = self.window.lookup_action(mirrored_action_name)
+		if args[1].get_string() == mirrored_action.get_state().get_string():
+			return False
+		self._enum_callback(*args)
+		self._mirror_lock = True
+		self._enum_callback(mirrored_action, args[1])
+		self._mirror_lock = False
+		return True
+
 	############################################################################
 
 	def persist_tools_options(self, *args):
@@ -112,10 +150,6 @@ class DrOptionsManager():
 			key_name = self._string_actions_from_gsetting[action_name]
 			self._persist_string(action_name, key_name)
 
-		# TODO faire une gaction pour ça !
-		font_fam_name = self.window.tools['text'].font_fam_name
-		self._tools_gsettings.set_string('last-font-name', font_fam_name)
-
 	def _persist_string(self, action_name, key_name):
 		action_value = self.get_value(action_name)
 		self._tools_gsettings.set_string(key_name, action_value)
@@ -156,9 +190,6 @@ class DrOptionsManager():
 			self._bottom_panes_dict[each_id].action_bar.set_visible(is_active)
 
 	def get_active_pane(self):
-		if self._active_pane_id is None:
-			return None # XXX encore des exceptions manuelles...
-			# return self._bottom_panes_dict['classic']
 		return self._bottom_panes_dict[self._active_pane_id]
 
 	def update_pane(self, tool):
@@ -201,7 +232,7 @@ class DrOptionsManager():
 	############################################################################
 	# Methods specific to the optionsbar for classic tools #####################
 
-	def get_classic_tools_pane(self): # XXX hardcoded
+	def get_classic_tools_pane(self):
 		return self._bottom_panes_dict['classic']
 
 	def left_color_btn(self):
@@ -232,6 +263,17 @@ class DrOptionsManager():
 	def get_left_color(self):
 		return self.left_color_btn().color_widget.get_rgba()
 
+	def get_persisted_color(self, is_left_color):
+		if is_left_color:
+			color_array = self._tools_gsettings.get_strv('last-left-rgba')
+		else:
+			color_array = self._tools_gsettings.get_strv('last-right-rgba')
+		r = float(color_array[0])
+		g = float(color_array[1])
+		b = float(color_array[2])
+		a = float(color_array[3])
+		return Gdk.RGBA(red=r, green=g, blue=b, alpha=a)
+
 	def get_operator(self):
 		# XXX répugnant, on duplique la donnée dans les 2 popovers, puis on
 		# hardcode qu'on prendra la donnée uniquement dans celui de droite, et
