@@ -1,6 +1,6 @@
-# color_select.py
+# select_color.py
 #
-# Copyright 2018-2021 Romain F. T.
+# Copyright 2018-2022 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,6 +16,9 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from .abstract_select import AbstractSelectionTool
+from .utilities_colors import utilities_get_rgba_name, \
+                              utilities_gdk_rgba_from_xy, \
+                              utilities_gdk_rgba_to_hexadecimal
 from .utilities_paths import utilities_get_magic_path
 
 class ToolColorSelect(AbstractSelectionTool):
@@ -27,19 +30,38 @@ class ToolColorSelect(AbstractSelectionTool):
 		# surrounding area made of white pixels.
 		super().__init__('color_select', _("Color selection"), 'tool-magic-symbolic', window)
 
+	def get_tooltip(self, event_x, event_y, motion_behavior):
+		color = utilities_gdk_rgba_from_xy(self.get_surface(), event_x, event_y)
+		if color is None:
+			return None
+		color_name = utilities_get_rgba_name(color)
+		color_code = utilities_gdk_rgba_to_hexadecimal(color)
+		return color_name + "\n" + color_code
+
+	def get_editing_tips(self):
+		tips = super().get_editing_tips()
+		if not self.selection_is_active():
+			label_warning1 = self.label + " - " + _("May not work for complex shapes")
+			tips.append(label_warning1)
+			label_warning2 = self.label + " - " + _("It will not work well " + \
+				                               "if the area's edges are blurry")
+			tips.append(label_warning2)
+		return tips
+
+	############################################################################
+
 	def press_define(self, event_x, event_y):
 		pass
 
-	def motion_define(self, event_x, event_y):
+	def motion_define(self, event_x, event_y, render):
 		pass
 
 	def release_define(self, surfc, event_x, event_y):
 		path = utilities_get_magic_path(surfc, event_x, event_y, self.window, 1)
-		self.get_selection().set_future_path(path)
+		self._pre_load_path(path)
 		if path is None:
 			return
 		self.operation_type = 'op-define'
-		self._set_future_coords_for_free_path()
 		operation = self.build_operation()
 		self.apply_operation(operation)
 
