@@ -1,6 +1,6 @@
-# free_select.py
+# select_free.py
 #
-# Copyright 2018-2021 Romain F. T.
+# Copyright 2018-2022 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,21 +31,27 @@ class ToolFreeSelect(AbstractSelectionTool):
 		self.add_tool_action_simple('selection_close', self._force_close_shape)
 		self.set_action_sensitivity('selection_close', False)
 
-	def on_tool_selected(self, *args):
-		super().on_tool_selected()
-
 	def on_tool_unselected(self, *args):
 		super().on_tool_unselected()
 		self.set_action_sensitivity('selection_close', False)
 
+	def get_editing_tips(self):
+		tips = super().get_editing_tips()
+		future_path = self.get_selection().get_future_path()
+		if future_path is not None and not self.selection_is_active():
+			label_instruction = self.label + " - " + \
+			                  _("Click on the shape's first point to close it.")
+			tips.append(label_instruction)
+		return tips
+
 	############################################################################
 
 	def press_define(self, event_x, event_y):
 		self._draw_shape(event_x, event_y)
 		self.set_action_sensitivity('selection_close', True)
 
-	def motion_define(self, event_x, event_y):
-		self._draw_shape(event_x, event_y)
+	def motion_define(self, event_x, event_y, render):
+		self._draw_shape(event_x, event_y, render)
 
 	def release_define(self, surface, event_x, event_y):
 		if self._draw_shape(event_x, event_y):
@@ -62,12 +68,14 @@ class ToolFreeSelect(AbstractSelectionTool):
 	def _force_close_shape(self, *args):
 		self.release_define(None, self.closing_x, self.closing_y)
 
-	def _draw_shape(self, event_x, event_y):
+	def _draw_shape(self, event_x, event_y, render=True):
 		"""This method is specific to the 'free selection' mode. It returns a
 		boolean, true if the shape should be closed."""
 		cairo_context = self.get_context()
 		cairo_context.set_source_rgba(0.5, 0.5, 0.5, 0.5)
-		cairo_context.set_dash([3, 3])
+		thickness = self.get_overlay_thickness()
+		cairo_context.set_dash([3 * thickness, 3 * thickness])
+		cairo_context.set_line_width(thickness)
 		if self.get_selection().get_future_path() is None:
 			self.closing_x = event_x
 			self.closing_y = event_y
@@ -83,10 +91,11 @@ class ToolFreeSelect(AbstractSelectionTool):
 			self._pre_load_path(cairo_context.copy_path())
 			return True
 		else:
-			cairo_context.line_to(int(event_x), int(event_y))
+			cairo_context.line_to(event_x, event_y)
 			cairo_context.stroke_preserve() # draw the line without closing the path
 			self._pre_load_path(cairo_context.copy_path())
-			self.non_destructive_show_modif()
+			if render:
+				self.non_destructive_show_modif()
 			return False
 
 	############################################################################
