@@ -1,6 +1,6 @@
 # tool_arc.py
 #
-# Copyright 2018-2021 Romain F. T.
+# Copyright 2018-2022 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,17 +26,17 @@ class ToolArc(AbstractClassicTool):
 		super().__init__('arc', _("Curve"), 'tool-arc-symbolic', window)
 		self.use_operator = True
 
-		self.add_tool_action_enum('line_shape', 'round')
-		self.add_tool_action_enum('dashes-type', 'none')
-		self.add_tool_action_enum('arrow-type', 'none')
-
 		# Default values
-		self._shape_label = _("Round")
 		self._cap_id = cairo.LineCap.ROUND
-
 		self._1st_segment = None
-		self._use_dashes = False
 		self._dashes_type = 'none'
+		self._arrow_type = 'none'
+		self._use_outline = False
+
+		self.add_tool_action_enum('line_shape', 'round')
+		self.add_tool_action_enum('dashes-type', self._dashes_type)
+		self.add_tool_action_enum('arrow-type', self._arrow_type)
+		self.add_tool_action_boolean('pencil-outline', self._use_outline)
 
 	def give_back_control(self, preserve_selection):
 		self._1st_segment = None
@@ -49,35 +49,55 @@ class ToolArc(AbstractClassicTool):
 		state_as_string = self.get_option_value('line_shape')
 		if state_as_string == 'thin':
 			self._cap_id = cairo.LineCap.BUTT
-			self._shape_label = _("Square")
 		else:
 			self._cap_id = cairo.LineCap.ROUND
-			self._shape_label = _("Round")
 
 	def get_options_label(self):
 		return _("Curve options")
 
-	def get_edition_status(self):
+	def get_editing_tips(self):
+		self._use_outline = self.get_option_value('pencil-outline')
 		self._dashes_type = self.get_option_value('dashes-type')
 		self._arrow_type = self.get_option_value('arrow-type')
 		self.set_active_shape()
 		is_arrow = self._arrow_type != 'none'
 		use_dashes = self._dashes_type != 'none'
-		label = self.label
+
+		label_segments = self.label + " - "
+		if self._1st_segment is not None:
+			label_segments += _("Draw a second segment to complete the curve")
+		else:
+			label_segments = None
+
+		label_options = self.label + " - "
 		if is_arrow and use_dashes:
-			label = label + ' - ' + _("Dashed arrow")
+			label_options += _("Dashed arrow")
 		elif is_arrow:
-			label = label + ' - ' + _("Arrow")
+			label_options += _("Arrow")
 		elif use_dashes:
-			label = label + ' - ' + _("Dashed")
-		return label
+			label_options += _("Dashed")
+		else:
+			label_options = None
+
+		if self.get_image().get_mouse_is_pressed():
+			label_modifier_alt = None
+		else:
+			label_modifier_alt = self.label + " - " + \
+			                     _("Press <Alt> to toggle the 'outline' option")
+
+		full_list = [label_segments, label_options, label_modifier_alt]
+		return list(filter(None, full_list))
 
 	############################################################################
 
 	def on_press_on_area(self, event, surface, event_x, event_y):
 		self.set_common_values(event.button, event_x, event_y)
 
-	def on_motion_on_area(self, event, surface, event_x, event_y):
+		self.update_modifier_state(event.state)
+		if 'ALT' in self._modifier_keys:
+			self._use_outline = not self._use_outline
+
+	def on_motion_on_area(self, event, surface, event_x, event_y, render=True):
 		cairo_context = self.get_context()
 		if self._1st_segment is None:
 			cairo_context.move_to(self.x_press, self.y_press)
@@ -87,6 +107,9 @@ class ToolArc(AbstractClassicTool):
 			cairo_context.curve_to(self._1st_segment[2], self._1st_segment[3], \
 			                       self.x_press, self.y_press, event_x, event_y)
 		self._path = cairo_context.copy_path()
+
+		if not render:
+			return
 		operation = self.build_operation(event_x, event_y)
 		self.do_tool_operation(operation)
 
@@ -95,7 +118,6 @@ class ToolArc(AbstractClassicTool):
 			self._1st_segment = (self.x_press, self.y_press, event_x, event_y)
 			return
 		else:
-			self.restore_pixbuf()
 			cairo_context = self.get_context()
 			cairo_context.move_to(self._1st_segment[0], self._1st_segment[1])
 			cairo_context.curve_to(self._1st_segment[2], self._1st_segment[3], \
@@ -112,12 +134,14 @@ class ToolArc(AbstractClassicTool):
 		operation = {
 			'tool_id': self.id,
 			'rgba': self.main_color,
+			'rgba2': self.secondary_color,
 			'antialias': self._use_antialias,
 			'operator': self._operator,
 			'line_width': self.tool_width,
 			'line_cap': self._cap_id,
-			'dashes_type': self._dashes_type,
-			'arrow_type': self._arrow_type,
+			'dashes': self._dashes_type,
+			'arrow': self._arrow_type,
+			'outline': self._use_outline,
 			'path': self._path,
 			'x_release': event_x,
 			'y_release': event_y,
@@ -129,16 +153,13 @@ class ToolArc(AbstractClassicTool):
 	def do_tool_operation(self, operation):
 		cairo_context = self.start_tool_operation(operation)
 
-		rgba = operation['rgba']
-		cairo_context.set_source_rgba(rgba.red, rgba.green, rgba.blue, rgba.alpha)
 		cairo_context.set_operator(operation['operator'])
-
 		line_width = operation['line_width']
 		cairo_context.set_line_width(line_width)
 		self.set_dashes_and_cap(cairo_context, line_width, \
-		                        operation['dashes_type'], operation['line_cap'])
+		                             operation['dashes'], operation['line_cap'])
 
-		if operation['arrow_type'] == 'double':
+		if operation['arrow'] == 'double':
 			for pts in operation['path']:
 				# how to do without a for???
 				if(pts[0] == cairo.PathDataType.MOVE_TO):
@@ -151,13 +172,20 @@ class ToolArc(AbstractClassicTool):
 
 		cairo_context.append_path(operation['path'])
 
-		if operation['arrow_type'] != 'none':
+		if operation['arrow'] != 'none':
 			x1 = operation['x_press']
 			y1 = operation['y_press']
 			x2 = operation['x_release']
 			y2 = operation['y_release']
 			utilities_add_arrow_triangle(cairo_context, x2, y2, x1, y1, line_width)
 
+		if operation['outline']:
+			cairo_context.set_source_rgba(*operation['rgba2'])
+			cairo_context.set_line_width(line_width * 1.2 + 2)
+			cairo_context.stroke_preserve()
+
+		cairo_context.set_line_width(line_width)
+		cairo_context.set_source_rgba(*operation['rgba'])
 		cairo_context.stroke()
 
 	############################################################################
