@@ -1,6 +1,6 @@
 # tool_paint.py
 #
-# Copyright 2018-2021 Romain F. T.
+# Copyright 2018-2022 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,14 +34,19 @@ class ToolPaint(AbstractClassicTool):
 	def get_options_label(self):
 		return _("Painting options")
 
-	def get_edition_status(self):
+	def get_editing_tips(self):
 		paint_algo = self.get_option_value('paint_algo')
 		if paint_algo == 'clipping':
-			return _("Click on an area to replace its color by transparency")
+			return [_("Click on an area to replace its color by transparency")]
 		elif paint_algo == 'whole':
-			return _("Click on the canvas to entirely paint it")
-		else:
-			return self.label
+			return [_("Click on the canvas to entirely paint it")]
+
+		label_warning1 = self.label + " - " + _("May not work for complex shapes")
+		label_warning2 = self.label + " - " + _("It will not work well if " + \
+		                                          "the area's edges are blurry")
+		return [label_warning1, label_warning2]
+
+	############################################################################
 
 	def on_press_on_area(self, event, surface, event_x, event_y):
 		self.set_common_values(event.button, event_x, event_y)
@@ -52,17 +57,18 @@ class ToolPaint(AbstractClassicTool):
 		or event_y < 0 or event_y > surface.get_height():
 			return
 
-		(x, y) = (int(event_x), int(event_y))
-		self.old_color = utilities_get_rgba_for_xy(surface, x, y)
+		self.old_color = utilities_get_rgba_for_xy(surface, event_x, event_y)
 
 		if self.get_option_value('paint_algo') == 'fill':
-			self._magic_path = utilities_get_magic_path(surface, x, y, self.window, 1)
+			self._magic_path = utilities_get_magic_path(surface, event_x, \
+			                                            event_y, self.window, 1)
 		elif self.get_option_value('paint_algo') == 'replace':
-			self._magic_path = utilities_get_magic_path(surface, x, y, self.window, 2)
+			self._magic_path = utilities_get_magic_path(surface, event_x, \
+			                                            event_y, self.window, 2)
 		else:
 			pass # == 'clipping' or == 'whole'
 
-		operation = self.build_operation(x, y)
+		operation = self.build_operation(event_x, event_y)
 		self.apply_operation(operation)
 
 	############################################################################
@@ -73,7 +79,7 @@ class ToolPaint(AbstractClassicTool):
 			'algo': self.get_option_value('paint_algo'),
 			# 'x': x,
 			# 'y': y,
-			'rgba': self.main_color,
+			'new_rgba': self.main_color,
 			'antialias': self._use_antialias,
 			'old_rgba': self.old_color,
 			'path': self._magic_path
@@ -97,8 +103,7 @@ class ToolPaint(AbstractClassicTool):
 	def _op_whole(self, operation):
 		"""Paint the entire image regardless of existing pixels"""
 		cairo_context = self.get_context()
-		rgba = operation['rgba']
-		cairo_context.set_source_rgba(rgba.red, rgba.green, rgba.blue, rgba.alpha)
+		cairo_context.set_source_rgba(*operation['new_rgba'])
 		cairo_context.paint()
 
 	def _op_fill(self, operation):
@@ -107,8 +112,7 @@ class ToolPaint(AbstractClassicTool):
 		if operation['path'] is None:
 			return
 		cairo_context = self.get_context()
-		rgba = operation['rgba']
-		cairo_context.set_source_rgba(rgba.red, rgba.green, rgba.blue, rgba.alpha)
+		cairo_context.set_source_rgba(*operation['new_rgba'])
 		cairo_context.append_path(operation['path'])
 		# can_fill = cairo_context.in_fill(operation['x'], operation['y'])
 		# print(can_fill)
@@ -137,7 +141,6 @@ class ToolPaint(AbstractClassicTool):
 		# First, everything BUT the targeted area is erased
 		surface = self.get_surface()
 		cairo_context = cairo.Context(surface)
-		rgba = operation['rgba']
 		old_rgba = operation['old_rgba']
 		cairo_context.set_source_rgba(255, 255, 255, 1.0)
 		cairo_context.append_path(operation['path'])
@@ -163,7 +166,7 @@ class ToolPaint(AbstractClassicTool):
 			self._replace_temp_with_alpha(red, green, blue)
 			i = i+1
 
-		# The main pixbuf is restored, since its alteration were a side effect
+		# The main pixbuf is restored, since its alterations were a side effect
 		# of this highly stupid algorithm
 		self.restore_pixbuf()
 		# We get a context from it again
@@ -188,7 +191,7 @@ class ToolPaint(AbstractClassicTool):
 
 		# The transparent pixels within the path are painted with the new color
 		cairo_context2.set_operator(cairo.Operator.DEST_OVER)
-		cairo_context2.set_source_rgba(rgba.red, rgba.green, rgba.blue, rgba.alpha)
+		cairo_context2.set_source_rgba(*operation['new_rgba'])
 		cairo_context2.append_path(operation['path'])
 		# cairo_context2.paint()
 		cairo_context2.set_line_width(3) # ptêt too much mdr
