@@ -1,6 +1,6 @@
 # tool_text.py
 #
-# Copyright 2018-2021 Romain F. T.
+# Copyright 2018-2022 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -60,7 +60,8 @@ class ToolText(AbstractClassicTool):
 
 		# for f in PangoCairo.font_map_get_default().list_families():
 		# 	print(f.get_name())
-		status = dialog.run()
+
+		status = dialog.run() # <<< FIXME
 		if(status == Gtk.ResponseType.OK):
 			self._font_fam_name = dialog.get_font_family().get_name()
 			# print(dialog.get_font())
@@ -68,6 +69,7 @@ class ToolText(AbstractClassicTool):
 			self.window.lookup_action('text-active-family').set_state(font_gvar)
 			self._preview_text()
 		dialog.destroy()
+		# TODO update le editing tip !!
 
 	def _set_font_options(self, *args):
 		# XXX incomplete? OBLIQUE exists
@@ -80,22 +82,25 @@ class ToolText(AbstractClassicTool):
 	def get_options_label(self):
 		return _("Text options")
 
-	def get_edition_status(self):
+	def get_editing_tips(self):
 		self._set_font_options()
-
 		self._set_background_style()
-		bg_label = {
-			'none': _("No background"),
-			'shadow': _("Shadow"),
-			'thin-outline': _("Thin outline"),
-			'thick-outline': _("Thick outline"),
-			'rectangle': _("Rectangle background"),
-		}[self._background_id]
 
-		# The current method is likely called because an option changed
+		# get_editing_tips is likely called because an option changed
 		self._force_refresh()
 
-		return self.label + ' - ' + self._font_fam_name + ' - ' + bg_label
+		label_options = self.label + " - " + self._font_fam_name
+		if self._background_id != 'none':
+			bg_label = {
+				'none': _("No background"),
+				'shadow': _("Shadow"),
+				'thin-outline': _("Thin outline"),
+				'thick-outline': _("Thick outline"),
+				'rectangle': _("Rectangle background"),
+			}[self._background_id]
+			label_options += " - " + bg_label
+
+		return [label_options]
 
 	############################################################################
 
@@ -114,7 +119,7 @@ class ToolText(AbstractClassicTool):
 			self._on_cancel()
 
 	def force_text_tool(self, string):
-		self.row.set_active(True)
+		self.select_flowbox_child()
 		self.set_common_values(self._last_click_btn, 100, 100)
 		self._open_popover_at(100, 100)
 		self._set_string(string)
@@ -130,7 +135,7 @@ class ToolText(AbstractClassicTool):
 		self._last_click_btn = event.button
 		self._should_cancel = True
 		self.set_common_values(self._last_click_btn, event_x, event_y)
-		self._open_popover_at(int(event.x), int(event.y))
+		self._open_popover_at(event.x, event.y)
 
 	# XXX could there be a better way to move the text ?
 	def _open_popover_at(self, x, y):
@@ -214,10 +219,9 @@ class ToolText(AbstractClassicTool):
 		font_fam = operation['font_fam']
 		font_size = operation['font_size'] * 2
 		entire_text = operation['text']
-		c1 = operation['rgba1']
-		c2 = operation['rgba2']
-		text_x = int(operation['x'])
-		text_y = int(operation['y'])
+		background_color = operation['rgba2']
+		text_x = operation['x']
+		text_y = operation['y']
 
 		font_description_string = font_fam
 		if operation['is_italic']:
@@ -245,15 +249,15 @@ class ToolText(AbstractClassicTool):
 			lines = entire_text.split('\n')
 			line_y = text_y
 			for line_text in lines:
-				line_y = self._op_bg_rectangle(cairo_context, layout, c2, \
-				                                      text_x, line_y, line_text)
+				line_y = self._op_bg_rectangle(cairo_context, layout, \
+				                    background_color, text_x, line_y, line_text)
 		elif operation['background'] == 'shadow':
 			dist = max(min(int(font_size/16), 4), 1)
-			cairo_context.set_source_rgba(c2.red, c2.green, c2.blue, c2.alpha)
+			cairo_context.set_source_rgba(*background_color)
 			self._show_text_at_coords(cairo_context, layout, entire_text, \
 			                                       text_x + dist, text_y + dist)
 		elif operation['background'] == 'thin-outline':
-			cairo_context.set_source_rgba(c2.red, c2.green, c2.blue, c2.alpha)
+			cairo_context.set_source_rgba(*background_color)
 			dist = min(int(font_size/16), 10)
 			dist = max(dist, 2)
 			for dx in range(-dist, dist):
@@ -264,7 +268,7 @@ class ToolText(AbstractClassicTool):
 			# these `for`s and this `if` should outline with an octogonal shape,
 			# which is close enough to a smooth round outline imho.
 		elif operation['background'] == 'thick-outline':
-			cairo_context.set_source_rgba(c2.red, c2.green, c2.blue, c2.alpha)
+			cairo_context.set_source_rgba(*background_color)
 			dist = int(font_size/10)
 			dist = max(dist, 2)
 			for dx in range(-dist, dist):
@@ -277,7 +281,7 @@ class ToolText(AbstractClassicTool):
 		########################################################################
 		# Draw text ############################################################
 
-		cairo_context.set_source_rgba(c1.red, c1.green, c1.blue, c1.alpha)
+		cairo_context.set_source_rgba(*operation['rgba1'])
 		self._show_text_at_coords(cairo_context, layout, entire_text, \
 		                                                         text_x, text_y)
 
@@ -293,7 +297,7 @@ class ToolText(AbstractClassicTool):
 
 	def _op_bg_rectangle(self, context, layout, c2, text_x, line_y, line_text):
 		# The text is first "displayed" in a transparent color…
-		context.set_source_rgba(0.0, 0.0, 0.0, 0.50)
+		context.set_source_rgba(0.0, 0.0, 0.0, 0.0)
 		self._show_text_at_coords(context, layout, line_text, text_x, line_y)
 		# …so we can get the size of the displayed line…
 		ink_rect, logical_rect = layout.get_pixel_extents()
@@ -303,7 +307,7 @@ class ToolText(AbstractClassicTool):
 		context.rel_line_to(logical_rect.width, 0)
 		context.rel_line_to(0, -1 * delta_y)
 		context.close_path()
-		context.set_source_rgba(c2.red, c2.green, c2.blue, c2.alpha)
+		context.set_source_rgba(*c2)
 		context.fill()
 		context.stroke()
 		# The returned value will be used as the vertical coord of the next line
